"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.useViewModel = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _react = require("react");

var _context4 = require("./context");

var _use_client = require("./use_client");

var useViewModel = function useViewModel(modelName, aggregateIds, stateChangeCallback, queryOptions) {
  var context = (0, _react.useContext)(_context4.ResolveContext);
  var client = (0, _use_client.useClient)();
  var viewModels = context.viewModels;
  var viewModel = viewModels.find(function (_ref) {
    var name = _ref.name;
    return name === modelName;
  });

  if (!viewModel) {
    throw Error("View model " + modelName + " not exist within context");
  }

  var closure = (0, _react.useMemo)(function () {
    return {
      state: viewModel.projection.Init ? viewModel.projection.Init() : null
    };
  }, []);
  var setState = (0, _react.useCallback)(function (state) {
    closure.state = state;
    stateChangeCallback(closure.state);
  }, []);
  var queryState = (0, _react.useCallback)( /*#__PURE__*/(0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee() {
    var result;
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return client.query({
              name: modelName,
              aggregateIds: aggregateIds,
              args: {}
            }, queryOptions);

          case 2:
            result = _context.sent;

            if (result) {
              setState(result.data);
            }

          case 4:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  })), []);
  var applyEvent = (0, _react.useCallback)(function (event) {
    setState(viewModel.projection[event.type](closure.state, event));
  }, []);
  var connect = (0, _react.useCallback)(function (done) {
    var asyncConnect = /*#__PURE__*/function () {
      var _ref3 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2() {
        var subscribe, subscription;
        return _regenerator["default"].wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return queryState();

              case 2:
                subscribe = client.subscribe(modelName, aggregateIds, function (event) {
                  return applyEvent(event);
                }, undefined, function () {
                  return queryState();
                });
                _context2.next = 5;
                return subscribe;

              case 5:
                subscription = _context2.sent;

                if (subscription) {
                  closure.subscription = subscription;
                }

                return _context2.abrupt("return", subscription);

              case 8:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      }));

      return function asyncConnect() {
        return _ref3.apply(this, arguments);
      };
    }();

    if (typeof done !== 'function') {
      return asyncConnect();
    }

    asyncConnect().then(function (result) {
      return done(null, result);
    })["catch"](function (error) {
      return done(error, null);
    });
    return undefined;
  }, []);
  var dispose = (0, _react.useCallback)(function (done) {
    var asyncDispose = /*#__PURE__*/function () {
      var _ref4 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3() {
        return _regenerator["default"].wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!closure.subscription) {
                  _context3.next = 3;
                  break;
                }

                _context3.next = 3;
                return client.unsubscribe(closure.subscription);

              case 3:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3);
      }));

      return function asyncDispose() {
        return _ref4.apply(this, arguments);
      };
    }();

    if (typeof done !== 'function') {
      return asyncDispose();
    }

    asyncDispose().then(function () {
      return done();
    })["catch"](function (error) {
      return done(error);
    });
    return undefined;
  }, []);
  return (0, _react.useMemo)(function () {
    return {
      connect: connect,
      dispose: dispose
    };
  }, []);
};

exports.useViewModel = useViewModel;
//# sourceMappingURL=use_view_model.js.map